@@ -2,7 +2,7 @@
  * MultiMail offline mail reader
  * protos for mysystem.cc
 
- Copyright (c) 1999 William McBrine <wmcbrine@clark.net> 
+ Copyright (c) 2001 William McBrine <wmcbrine@users.sourceforge.net>
 
  Distributed under the GNU General Public License.
  For details, see the file COPYING in the parent directory. */
@@ -10,30 +10,42 @@
 #ifndef MYSYSTEM_H
 #define MYSYSTEM_H
 
-#include <ctime>
-
 extern "C" {
 #include <sys/types.h>
 }
 
+class mystat;
+
 char *myfgets(char *, size_t, FILE *);
 int mysystem(const char *);
-void mytmpnam(char *);
+int mysystem2(const char *, const char *);
+char *mytmpnam();
 void edit(const char *);
 int mychdir(const char *);
 int mymkdir(const char *);
 void myrmdir(const char *);
-void mygetcwd(char *);
-bool readable(const char *);
-bool writeable(const char *);
+char *mygetcwd();
 const char *sysname();
 bool myopendir(const char *);
-const char *myreaddir();
+const char *myreaddir(mystat &);
 void clearDirectory(const char *);
+time_t touchFile(const char *);
 
-#if defined (__MSDOS__) || defined (__EMX__)
-const char *canonize(const char *);
+#ifdef LIMIT_MEM
+long limitmem(long);
+#else
+# define limitmem(x) x
+#endif
 
+char *canonize(char *);
+
+#ifdef HAS_HOME
+const char *homify(const char *);
+#else
+# define homify(x) x
+#endif
+
+#ifdef USE_SHELL
 class Shell
 {
 	char *prompt;
@@ -42,25 +54,31 @@ class Shell
 	~Shell();
 	void out();
 };
-
-#else
-# define canonize(x) x
 #endif
 
 class mystat
 {
- public:
+	int mode;
 	off_t size;
-	time_t date;
-	bool isdir;
+	time_t date, adate;
+ public:
+	mystat(const char *);
+	mystat();
 
 	bool init(const char *);
+#ifdef USE_FINDFIRST
+	void init(long, long, char);
+#endif
+	void init();
+	bool isdir();
+	bool readable();
+	bool writeable();
+	off_t fsize();
+	time_t fdate();
+	void reset_date(const char *);
 };
 
-// Some of the functions normally used by MultiMail don't exist in EMX,
-// but are available under other names:
-
-#ifdef __EMX__
+#ifdef USE_STRICMP
 # define strcasecmp stricmp
 # define strncasecmp strnicmp
 #endif
